From c33e547fc38f1c8ddb49addb76aa6fc97407a0f6 Mon Sep 17 00:00:00 2001
From: John Arbuckle <programmingkidx@gmail.com>
Date: Thu, 28 Dec 2017 14:54:04 -0500
Subject: [PATCH] add support for ms and us words

Signed-off-by: John Arbuckle <programmingkidx@gmail.com>
---
 arch/ppc/qemu/init.c | 69 ++++++++++++++++++++++++++++++++++++++++++++++++++++
 1 file changed, 69 insertions(+)

diff --git a/arch/ppc/qemu/init.c b/arch/ppc/qemu/init.c
index 5ce080c..bbb2c4e 100644
--- a/arch/ppc/qemu/init.c
+++ b/arch/ppc/qemu/init.c
@@ -751,6 +751,73 @@ static void adler32(void)
     RET(s2 << 16 | s1);
 }
 
+/* delays execution for the specified time in microseconds ( us -- ) */
+static void microseconds(void)
+{
+    uint32_t us_delay; // user specified delay time
+    uint32_t multiplier;
+    uint32_t start_tbl, start_tbu;
+    uint32_t end_tbl, end_tbu;
+    uint32_t current_tbl, current_tbu;
+    
+    multiplier = 100;
+    us_delay = POP();
+    asm volatile("mftb %0" : "=r"(start_tbl));
+    asm volatile("mftbu %0" : "=r"(start_tbu));
+    end_tbl = start_tbl + us_delay * multiplier;
+    end_tbu = start_tbu;
+    if (end_tbl < start_tbl) { // if a low-order to high-order carry happened
+        end_tbu++;
+    }
+    
+    while (1) {
+        asm volatile("mftb %0" : "=r"(current_tbl));
+        asm volatile("mftbu %0" : "=r"(current_tbu));
+        if (end_tbu == current_tbu) {
+            if ((end_tbl - current_tbl) <= 0 ) {
+                return;
+            }
+        }
+        else if (end_tbu < current_tbu) {
+            return;
+        }
+    }
+}
+
+
+/* delays execution for the specified time in milliseconds ( ms -- ) */
+static void milliseconds(void)
+{
+    uint32_t ms_delay; // user specified delay time
+    uint32_t multiplier;
+    uint32_t start_tbl, start_tbu;
+    uint32_t end_tbl, end_tbu;
+    uint32_t current_tbl, current_tbu;
+    
+    multiplier = 100000;
+    ms_delay = POP();
+    asm volatile("mftb %0" : "=r"(start_tbl));
+    asm volatile("mftbu %0" : "=r"(start_tbu));
+    end_tbl = start_tbl + ms_delay * multiplier;
+    end_tbu = start_tbu;
+    if (end_tbl < start_tbl) { // if a low-order to high-order carry happened
+        end_tbu++;
+    }
+
+    while (1) {
+        asm volatile("mftb %0" : "=r"(current_tbl));
+        asm volatile("mftbu %0" : "=r"(current_tbu));
+        if (end_tbu == current_tbu) {
+            if ((end_tbl - current_tbl) <= 0 ) {
+                return;
+            }
+        }
+        else if (end_tbu < current_tbu) {
+            return;
+        }
+    }
+}
+
 void
 arch_of_init(void)
 {
@@ -1022,4 +1089,6 @@ arch_of_init(void)
     
     bind_func("platform-boot", boot);
     bind_func("(arch-go)", arch_go);
+    bind_func("ms", milliseconds);
+    bind_func("us", microseconds);
 }
-- 
2.14.3 (Apple Git-98)

